<?php

declare(strict_types=1);

namespace Drupal\ckeditor5_link_popup\Plugin\CKEditor5Plugin;

use Drupal\ckeditor5\HTMLRestrictions;
use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableInterface;
use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableTrait;
use Drupal\ckeditor5\Plugin\CKEditor5PluginDefault;
use Drupal\ckeditor5\Plugin\CKEditor5PluginElementsSubsetInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\EditorInterface;

/**
 * CKEditor 5 Link Popup Styles plugin.
 */
class LinkStyles extends CKEditor5PluginDefault implements CKEditor5PluginConfigurableInterface, CKEditor5PluginElementsSubsetInterface {

  use CKEditor5PluginConfigurableTrait;

  

  /**
   * @file
   * Contains the LinkStyles class.
   *
   * This class defines the default attributes for the link popup in CKEditor5.
   */

  

  protected $defaultAttributes = [
    'class' => 'use-ajax',
    'id' => 'link-modal',
    'data-dialog-options' => '{"width": 800, "dialogClass": "custom-class"}',
    'data-dialog-type' => 'modal',
  ];


  /**
   * Returns an array of elements subset.
   *
   * This method checks if the 'styles' configuration is empty. If it is empty,
   * an empty array is returned. Otherwise, an array containing the elements
   * '<a href id class data-*>' is returned.
   *
   * @return array
   *   An array of elements subset.
   */
  public function getElementsSubset(): array {
    if (empty($this->configuration['styles'])) {
      return [];
    }
    return ['<a href id class data-*>'];
  }

 
  /**
   * Provides the default configuration for the LinkStyles plugin.
   *
   * @return array
   *   An array containing the default configuration for the LinkStyles plugin.
   */
  public function defaultConfiguration() {
    return [
      'styles' => [
        [
          'label' => (string) $this->t('Link Modal'),
          'element' => '<a class="use-ajax">',
          'attributes' => $this->defaultAttributes,
        ],
      ],
    ];
  }

  /**
   * Builds the configuration form for the LinkStyles plugin.
   *
   * @param array $form
   *   An associative array containing the form elements.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form array with the configuration elements.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['styles'] = [
      '#title' => $this->t('Link Popup Styles'),
      '#type' => 'textarea',
      '#description' => $this->t('Define Link Popup Styles in JSON format, with "label", "element", and "attributes". Example: {"label": "Modal Link", "element": "a.use-ajax", "attributes": {"class": "use-ajax", "id": "link-modal", "data-dialog-options": "{\"width\": 800, \"dialogClass\": \"custom-class\"}", "data-dialog-type": "modal"}}.'),
      '#default_value' => json_encode($this->configuration['styles'], JSON_PRETTY_PRINT),
    ];

    return $form;
  }

 
 
  /**
   * Validates the configuration form for the LinkStyles plugin.
   *
   * This method is responsible for validating the JSON configuration provided
   * in the form. It checks if the JSON is correctly formatted and sets an error
   * message if it is not. It also sets the validated styles value in the form
   * state for further processing.
   *
   * @param array &$form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return void
   *   Returns nothing.
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $form_value = $form_state->getValue('styles');
    $styles = json_decode($form_value, true);

    if (json_last_error() !== JSON_ERROR_NONE) {
      $form_state->setError($form['styles'], $this->t('The JSON provided is invalid. Please make sure it is correctly formatted.'));
      return;
    }

    // Further validation could be added to check for specific required fields.
    $form_state->setValue('styles', $styles);
  }


  /**
   * Implements submitConfigurationForm() method for the LinkStyles plugin.
   *
   * This method is called when the configuration form for the plugin is submitted.
   * It updates the 'styles' configuration value with the submitted form value.
   *
   * @param array &$form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['styles'] = $form_state->getValue('styles');
  }

  /**
   * Retrieves the dynamic plugin configuration based on the static plugin configuration and the editor.
   *
   * @param array $static_plugin_config The static plugin configuration.
   * @param EditorInterface $editor The editor interface.
   * @return array The dynamic plugin configuration.
   */
  public function getDynamicPluginConfig(array $static_plugin_config, EditorInterface $editor): array {
    // Check if the 'styles' configuration is empty.
    if (empty($this->configuration['styles'])) {
      return [];
    }

    $config = [];
    // Iterate through each style in the 'styles' configuration.
    foreach ($this->configuration['styles'] as $style) {
      // Get the attributes for the style, or use the default attributes if not provided.
      $attributes = $style['attributes'] ?? $this->defaultAttributes;

      // Add the style as a decorator for the 'link' element in the configuration.
      $config['link']['decorators'][] = [
        'mode' => 'manual',
        'label' => $style['label'],
        'attributes' => $attributes,
      ];
    }

    return $config;
  }
}
